<?php

namespace Drupal\dsfr\settings;

use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Url;
use Drupal\dsfr\settings\Components;
use Drupal\dsfr\settings\Tools;

class Theme {

  /**
   * Tabs for theme settings forms
   *
   * @param string $theme
   * @return array<string, mixed>
   */
  public static function settings( string $theme = 'dsfr' ): array {

    $base_url = Url::fromRoute('<front>')->setAbsolute()->toString();
    $theme_current = \Drupal::config('system.theme')->get('default');
    $theme_path_current = \Drupal::theme()->getActiveTheme()->getPath();
    $version = Tools::getPackageVersion($theme_path_current);
    $theme_url_current = $base_url . $theme_path_current;
    $dist_current = $theme_url_current . '/dist/';    
    $theme_name = \Drupal::theme()->getActiveTheme()->getName();
    $theme_setting = \Drupal::config($theme_name . '.settings');

    // -------------------------------------------------------------------------------- //
    // Is this theme in a distribution ?
    $distribution = \Drupal::service('extension.list.profile')->getList();
    $is_kickstart = (array_key_exists('dsfr_kickstart', $distribution)) ? 1 : 0;      
 
    // -------------------------------------------------------------------------------- //
    // DSRF librarie
    $external_lib = $theme_setting->get('code.external_lib');
    // Components
    $dsfr = Components::initDsfrComponents($theme_setting, $external_lib);
        
    // -------------------------------------------------------------------------------- //
    // Legacy for Child theme
    $theme_path = $base_url . 'themes/contrib/dsfr';
    $dist = $theme_path . '/dist/';

    // -------------------------------------------------------------------------------- //
    // Get MAIN url
    $https = (isset($_SERVER['HTTP_X_FORWARDED_PROTO']) && $_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https' && $_SERVER["REQUEST_SCHEME"] == 'http') ? 'https://' : 'http://';

    // -------------------------------------------------------------------------------- //
    // Default container
    $container = 'fr-container';

    // -------------------------------------------------------------------------------- //
    // Breadcrumb position
    $breadcrumb_position = self::getBreadcrumbPosition($theme_setting);

    // -------------------------------------------------------------------------------- //

    $important_message = ($theme_setting->get('info_important.message')) ? $theme_setting->get('info_important.message') : '';
    
    // -------------------------------------------------------------------------------- //
    // Final result
    return self::buildFinalResult(
      $version, 
      $https, 
      $theme_current, $theme_path_current, $theme_url_current, $dist_current, $dist, $theme, $theme_path, 
      $dsfr, $external_lib, 
      $is_kickstart, 
      $theme_setting, 
      $container, 
      $breadcrumb_position, 
      $important_message
    );
  }

  /**
   * Get breadcrumb position
   *
   * @param ImmutableConfig $theme_setting
   * @return string
   */
  private static function getBreadcrumbPosition(ImmutableConfig $theme_setting): string {
    
    switch ($theme_setting->get('breadcrumb.position')) {
      case '1': return ' rf-breadcrumb--center';
      case '2': return ' rf-breadcrumb--right';
      default: return '';
    }
  }

  /**
   * Build final result array
   *
   * @param string $version
   * @param string $https
   * @param string $theme_current
   * @param string $theme_url_current
   * @param string $dist_current
   * @param string $dist
   * @param string $theme
   * @param string $theme_path
   * @param array<string, mixed> $dsfr
   * @param mixed $external_lib
   * @param int $is_kickstart
   * @param ImmutableConfig $theme_setting
   * @param string $container
   * @param string $breadcrumb_position
   * @param string $important_message
   * @return array<string, mixed>
   */
  private static function buildFinalResult(
    string $version,
    string $https,
    string $theme_current,
    string $theme_path_current,
    string $theme_url_current,
    string $dist_current,
    string $dist,
    string $theme,
    string $theme_path,
    array $dsfr,
    $external_lib,
    int $is_kickstart,
    ImmutableConfig $theme_setting,
    string $container,
    string $breadcrumb_position,
    string $important_message
  ): array {

    $domain = $https . $_SERVER['HTTP_HOST'];
    $artwork = '/' . $theme_path_current . '/dist/dsfr/artwork/';

    return [

      'version'   => $version,
      'domain'    => $domain,
      'name'      => $theme_current,
      'path'      => $theme_path_current,
      'url'       => $theme_url_current,
      'settings'  => Url::fromRoute('system.theme_settings_theme', ['theme' => $theme_current])->toString(),
      'dist'      => $dist_current,
      'favicons'  => $dist_current. 'dsfr/favicon/',
      'background'=> $artwork . 'background/',
      'pictogram' => $artwork . 'pictograms/',
      'img'       => $dist_current. 'img/',
      'lib'       => $dist . 'dsfr/',
      'dsfr'      => [
        'name'    => $theme,
        'path'    => $theme_path,
        'dist'    => $dist,
        'favicons'=> $dist . 'dsfr/favicon/',
        'img'     => $dist . 'img/',
        'lib'     => $dist . 'dsfr/',
        'components' => $dsfr['components'],
        'external'=> $external_lib,
        'kickstart'=> $is_kickstart
      ],
      'general'   => [
        'institution' => ($theme_setting->get('general.institution')) ? $theme_setting->get('general.institution') : 'République <br> Française'
      ],
      'header'    => [
        'container' => ($theme_setting->get('header.container')) ? $container . '-fluid' : $container,
        'logo'    => [
          'img'   => $theme_setting->get('header.logo.img'),
          'text'   => $theme_setting->get('header.logo.text')
        ],
        'settings' => $theme_setting->get('header.settings'),     
      ],
      'navigation'  => [
        'container' => ($theme_setting->get('navigation.container')) ? $container . '-fluid' : $container,
      ],
      'info_important' => [
        'message' => $important_message,
        'show' => $theme_setting->get('info_important.show'),
        'close' => $theme_setting->get('info_important.close')
      ],
      'hero' => [
        'container' => ($theme_setting->get('hero.container')) ? $container . '-fluid' : $container,
        'margin' => [
          'top'     => $theme_setting->get('hero.margin.top'),
          'bottom'  => $theme_setting->get('hero.margin.bottom')
        ]
      ],
      'breadcrumb' => [
        'position'  => $breadcrumb_position,
        'margin' => [
          'top'     => $theme_setting->get('breadcrumb.margin.top'),
          'bottom'  => $theme_setting->get('breadcrumb.margin.bottom')
        ]
      ],
      'content' => [
        'container' => ($theme_setting->get('content.container')) ? $container . '-fluid' : $container,
        'margin' => [
          'top'     => $theme_setting->get('content.margin.top'),
          'bottom'  => $theme_setting->get('content.margin.bottom')
        ]
      ],
      'footer_top' => [
        'container' => ($theme_setting->get('footer_top.container')) ? $container . '-fluid' : $container,
      ],
      'footer' => [
        'container' => ($theme_setting->get('footer.container')) ? $container . '-fluid' : $container,
        'logo' => [
          'img' => $theme_setting->get('footer.logo.img')
        ],
        'last_line' => $theme_setting->get('footer.last_line')
      ], 
      'code'   => [
        'gdpr' => $theme_setting->get('code.gdpr'),
        'newsletter' => $theme_setting->get('code.newsletter'),
        'w3c' => $theme_setting->get('code.w3c')
      ],
      'features' => [
        'node_user_picture' => $theme_setting->get('features.node_user_picture'),
        'comment_user_picture' => $theme_setting->get('features.comment_user_picture'),
        'comment_user_verification' => $theme_setting->get('features.comment_user_verification'),
        'favicon' => $theme_setting->get('features.favicon')
      ],
      'logo' => [
        'use_default' => $theme_setting->get('logo.use_default'),
        'path'        => $theme_setting->get('logo.path')
      ],
      'favicon' => [
        'use_default' => $theme_setting->get('favicon.use_default'),
        'path'        => $theme_setting->get('favicon.path')
      ]
    ];    
  }

}